"use client"

import React from 'react'
import Image from 'next/image'
import Link from 'next/link'
import { deleteProperties, getProperties, updatePrice, updateStatus } from '@/actions/bmv'
import DataTable from '@/components/table/data-table'
import { Button } from '@/components/ui/button'
import { ColumnDef } from '@tanstack/react-table'
import { BathIcon, BedIcon, ConstructionIcon, CopyIcon, Edit3Icon, Search, User2, XIcon } from 'lucide-react'
import { toast } from 'sonner'
import { Switch } from '@/components/ui/switch'
import {
    Card,
    CardContent,
    CardDescription,
    CardHeader,
    CardTitle,
} from "@/components/ui/card"
import { Input } from '@/components/ui/input'
import { formatCurrency } from '@/lib/format-currency'
import { useLoading } from '@/lib/useLoading'
import DeleteDialog from '@/components/delete/delete-dialog'
import UpdatePriceDialog from './update-price-dialog'
import { Badge } from '@/components/ui/badge'
import { useDebounce } from '@/hooks/useDebounce'
import moment from 'moment-timezone'
import { useQuery } from '@tanstack/react-query'
import { toasty } from '@/utils/toasty'

const ListVillas = () => {
    const [offset, setOffset] = React.useState(0);
    const [search, setSearch] = React.useState('');
    const {loading, setLoading} = useLoading();
    const debSearch = useDebounce(search, 500);

    const { data, refetch, isPending , isError, error, isLoading} = useQuery({
        queryKey: ['bmv-villas', debSearch, offset],
        queryFn: async () => {
            const response = await getProperties(offset, debSearch);
            if (!response.success) {
                throw new Error(response.message);
            }
            return response;
        },
        refetchInterval: 60 * 1000,
        refetchOnWindowFocus: false
    });

    React.useEffect(() => {
        if (isError && error) {
            toast.error(error.message || "Failed to fetch data");
        }
        if (loading && !isLoading) {
            setLoading(false);
        }
    }, [isError, error, loading, isLoading]);

    const villas = data?.data || [];
    const hasMore = data?.hasMore || false;
    const page = {
        page: data?.currentPage || 1,
        allPage: data?.totalPages || 1,
        total: data?.total || 0,
    };

    const handleSearchChange = (e: React.ChangeEvent<HTMLInputElement>) => {
        setSearch(e.target.value);
        setOffset(0)
    };

    const nextPage = () => {
        if (hasMore) setOffset(prev => prev + 10);
    };

    const prevPage = () => {
        setOffset(prev => Math.max(0, prev - 10));
    };

    const clearSearch = () => {
        setSearch('');
        setOffset(0);
    };

    const columns: ColumnDef<any>[] = [
        {
            accessorFn: (row: any) => row._id,
            id: 'id',
            header: () => <div className='flex items-center justify-center w-full font-bold'>ID</div>,
            enableSorting: true,
            enableColumnFilter: false,
            enableGlobalFilter: true,
            cell: (info: any) => {
                const row = info.row.original;
                return (
                    <div className='flex items-center justify-center'>
                        <Button size="sm" variant="ghost" onClick={() => copyToClipboard(row._id)}>
                            <CopyIcon />
                        </Button>
                    </div>
                )
            },
        },
        {
            accessorKey: 'propertyId',
            header: () => <span className='font-bold'>PROPERTY ID</span>,
            enableSorting: true,
            enableColumnFilter: false,
            enableGlobalFilter: true,
            cell: (info: any) => {
                const row = info.row.original;
                return (
                    <div className='flex items-center'>
                        <p className='w-fit text-[13px]'>{row.propertyId}</p>
                        <Button size="sm" variant="ghost" onClick={() => copyToClipboard(row.propertyId)}>
                            <CopyIcon />
                        </Button>
                    </div>
                )
            },
        },
        {
            accessorKey: 'roomId',
            header: () => <span className='font-bold'>ROOM ID</span>,
            enableSorting: true,
            enableColumnFilter: false,
            enableGlobalFilter: true,
            cell: (info: any) => {
                const row = info.row.original;
                return (
                    <div className='flex items-center'>
                        <p className='w-fit text-[13px]'>{row.roomId}</p>
                        <Button size="sm" variant="ghost" onClick={() => copyToClipboard(row.roomId)}>
                            <CopyIcon />
                        </Button>
                    </div>
                )
            },
        },           
        {
            accessorFn: (row: any) => row.name,
            id: 'name',
            header: () => <span className='font-bold'>VILLA NAME</span>,
            enableSorting: true,
            enableColumnFilter: false,
            enableGlobalFilter: true,
            cell: (info: any) => {
                const row = info.row.original;
                return <p className='w-fit font-medium'>{row.name}</p>
            },
        },
        {
            accessorKey: 'address',
            id: 'address',
            header: () => <span className='font-bold'>VILLA ADDRESS</span>,
            enableSorting: true,
            enableColumnFilter: false,
            enableGlobalFilter: true,
            cell: (info: any) => {
                const row = info.row.original;
                return (
                    <div className='flex items-center gap-2'>
                        <p className='w-fit line-clamp-1'>{row.address ? `${row.address.slice(0, 30)}...` : "No Address"}</p>
                        <Badge className='w-fit' variant="secondary">
                            <span>{row.city}</span>
                        </Badge>
                    </div>
                )
            },
        },
        {
            accessorFn: (row: any) => row.guest || row.bedroom || row.bathroom,
            accessorKey: 'additional',
            header: () => <div className='flex items-center justify-center w-full font-bold'>FACILITIES</div>,
            enableSorting: true,
            enableColumnFilter: false,
            enableGlobalFilter: true,
            cell: (info: any) => {
                const row = info.row.original;
                return (
                    <div className='flex items-center justify-center gap-3'>
                        <Badge className='w-14' variant="blue">
                            <User2 />
                            <span>{row.guest}</span>
                        </Badge>
                        <Badge className='w-14' variant="blue">
                            <BedIcon />
                            <span>{row.bedroom}</span>
                        </Badge>
                        <Badge className='w-14' variant="blue">
                            <BathIcon />
                            <span>{row.bedroom}</span>
                        </Badge>
                    </div>
                )
            },
        },
        {
            accessorKey: 'construction',
            header: () => <div className='flex text-yellow-500 items-center justify-center w-full'><ConstructionIcon /></div>,
            enableSorting: false,
            enableColumnFilter: false,
            enableGlobalFilter: true,
            cell: (info: any) => {
                const row = info.row.original;
                return (
                    <div className='flex items-center justify-center'>
                        <Switch 
                            checked={row.construction} 
                            onCheckedChange={(value) => handleStatus(row._id, "construction", value)}
                            className='data-[state=checked]:bg-gundam-green data-[state=unchecked]:bg-gundam-red'
                        />
                    </div>
                )
            },
        },
        {
            accessorKey: 'dailyPriceUSD',
            header: () => <span className='font-bold'>PRICE</span>,
            enableSorting: true,
            enableColumnFilter: false,
            enableGlobalFilter: true,
            cell: (info: any) => {
                const row = info.row.original;
                return <p className="w-fit before:content-['$'] before:mr-1 font-bold">{formatCurrency(row.dailyPriceUSD)}</p>
            },
        },
        {
            accessorKey: 'status',
            header: () => <span className='font-bold'>MANAGED</span>,
            enableSorting: true,
            enableColumnFilter: false,
            enableGlobalFilter: true,
            cell: (info: any) => {
                const row = info.row.original;
                return (
                    <div className='flex items-center justify-center'>
                        <Switch 
                            checked={row.status} 
                            onCheckedChange={(value) => handleStatus(row._id, "status", value)}
                            className='data-[state=checked]:bg-gundam-green data-[state=unchecked]:bg-gundam-red'
                        />
                    </div>
                )
            },
        },
        {
            accessorKey: 'updatedAt',
            header: () => <span className='font-bold'>LAST UPDATE</span>,
            enableSorting: true,
            enableColumnFilter: false,
            enableGlobalFilter: true,
            cell: (info: any) => {
                const row = info.row.original;
                return <p className='w-fit text-sm'>{moment(row.updatedAt).tz("Asia/Makassar").fromNow()}</p>
            },
        },
        {
            accessorKey: 'action',
            header: () => <div className='flex items-center justify-center w-full font-bold'>ACTIONS</div>,
            enableSorting: false,
            enableColumnFilter: false,
            cell: (info: any) => {
                const row = info.row.original;
                return (
                    <div className='flex items-center justify-center gap-2'>
                        <UpdatePriceDialog 
                            onAction={() => handlePrice(row._id)}
                        />
                        <Button size="sm" variant="blue" asChild>
                            <Link href={`/dashboard/bmv/villas/${row._id}`}>
                                <Edit3Icon className="h-4 w-4" />
                            </Link>
                        </Button>
                        <DeleteDialog 
                            onDelete={() => handleDelete(row._id)}
                        />
                    </div>
                );
            },
        },
    ]

    const handleDelete = async (id: string) => {
        if (!id) return;
        toasty(deleteProperties(id),{
            loading() {
                setLoading(true)
                return `Deleting villa...`
            },
            success(data) {
                refetch();
                return data.message
            },
            error(err) {
                return err.message || "Failed to delete villa"
            },
            finally() {
                setLoading(false)
            },
        })
    }
    const handleStatus = async (id: string, key:string, value: any) => {
        if (!id || !key) return;
        
        const body = {
            key,
            value
        }

        toasty(updateStatus(id, body),{
            loading() {
                setLoading(true)
                return `Update ${key}...`
            },
            success(data) {
                refetch();
                return data.message
            },
            error(err) {
                return err.message || `Failed to update ${key}`
            },
            finally() {
                setLoading(false)
            },
        })
    }

    const handlePrice = async (id: string) => {
        if (!id) return;
        toasty(updatePrice(id),{
            loading() {
                setLoading(true)
                return `Update price...`
            },
            success(data) {
                refetch();
                return data.message
            },
            error(err) {
                return err.message || "Failed to update price"
            },
            finally() {
                setLoading(false)
            },
        })
    }

    const copyToClipboard = (text: string) => {
        navigator.clipboard
        .writeText(text)
        .then(() => {
            toast.success("Copied to clipboard!");
        })
        .catch(() => {
            toast.error("Failed to copy!");
        });
    };

    return (
        <div>
            <Card>
                <CardHeader>
                    <CardTitle>Explore All Villas</CardTitle>
                    <CardDescription>Browse through our curated collection of villas available for booking in Bali, managed by Bali Management Villas.</CardDescription>
                </CardHeader>
                <CardContent className='space-y-6'>
                    <div className='flex items-center justify-between'>
                        <div className='flex items-center gap-3'>
                            <div className="relative max-w-2xs">
                                <Input
                                    type="text"
                                    value={search}
                                    placeholder="Search villas..."
                                    className="border px-2 py-1 rounded-lg"
                                    onChange={handleSearchChange}
                                />
                                {search && (
                                    <div className="absolute top-2 right-2 translate translate-y-0">
                                        <Button 
                                            type="button" 
                                            variant="black" 
                                            size="bullet" 
                                            onClick={clearSearch}
                                        >
                                            <XIcon />
                                        </Button>
                                    </div>
                                )}
                            </div>
                            <Button 
                                size="rounded"
                                onClick={() => {
                                    if(!search) return;
                                    refetch()
                                }}
                            >
                                <Search />
                            </Button>
                        </div>
                        <div className="flex items-center gap-3">
                            <Button variant="semi" asChild>
                                <Link href="/dashboard/bmv/villas/add" onNavigate={() => setLoading(true)}>
                                    Add Data
                                </Link>
                            </Button>
                            <Button variant="green" onClick={() => refetch()}>
                                Refresh Data
                            </Button>
                        </div>
                    </div>
                    <DataTable 
                        data={villas}
                        columns={columns}
                        isLoading={isPending || isLoading}
                        next={nextPage}
                        prev={prevPage}
                        canNextpage={!hasMore}
                        canPrevPage={offset === 0}
                        page={page?.page || 0}
                        allPage={page?.allPage || 0}
                        total={page?.total || 0}
                    />
                </CardContent>
            </Card>
        </div>
    )
}

export default ListVillas